#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016.03.24
#
# Monitors Cisco ASA connection count
#
# works with Cisco ASA 55xx and later, tested witch 5506W and 5512-X, 5585-SSP2
#
# 2016-03-24: changed "snmp_scan_function" from ".1.3.6.1.2.1.1.2.0" to ".1.3.6.1.2.1.1.1.0"
# 2016-07-02: fixed crit/warn to >=
# 2018-01-09: some fine tuning (changed infotext)
# 2020-02-13: changed snmp_info from ".1.3.6.1.4.1.9.9.147.1.2.2.2.1.5" to
#             more specific "1.3.6.1.4.1.9.9.147.1.2.2.2.1.5.40"
# 2020-02-24: added support for Cisco Firepower Threat Defense
# 2020-04-28: changed item from Cisco ASA connections to Firewall connections --> more clear, with cisco_asa_conn check
# 2021-03-24: rewrite for CMK 2.0 Check API, added lower levels
# 2023-06-07: fixed IndexError (list index out of range) in parse function
#

# sample snmp walk
#
# .1.3.6.1.4.1.9.9.147.1.2.2.2.1.5.40.6 = Gauge32: 4987
# .1.3.6.1.4.1.9.9.147.1.2.2.2.1.5.40.7 = Gauge32: 17517
#
# CISCO-FIREWALL-MIB::cfwConnectionStatValue.protoIp.currentInUse = Gauge32: 4987
# CISCO-FIREWALL-MIB::cfwConnectionStatValue.protoIp.high = Gauge32: 17517
#
#

from typing import NamedTuple, Optional

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    check_levels,
    SNMPTree,
    startswith,
    contains,
    any_of,
)


class CiscoAsaFwConnections(NamedTuple):
    current: int
    peak: int


# [['1102', '2290']]
def parse_cisco_asa_connections(string_table: StringTable) -> Optional[CiscoAsaFwConnections]:
    try:
        current, peak = string_table[0]
    except (IndexError, ValueError):
        return

    return CiscoAsaFwConnections(
        current=int(current),
        peak=int(peak)
    )


#  CiscoAsaFwConnections(current=1102, peak=2290)
def discovery_cisco_asa_connections(section: CiscoAsaFwConnections) -> DiscoveryResult:
    yield Service()


def check_cisco_asa_connections(params, section: CiscoAsaFwConnections) -> CheckResult:
    yield from check_levels(
        section.current,
        levels_lower=params.get('connections_lower', None),
        levels_upper=params.get('connections', None),
        metric_name='fw_connections_active',
        render_func=lambda v: '%s' % str(v),
    )

    yield from check_levels(
        section.peak,
        label='Max. since system startup',
        metric_name='fw_connections_peak',
        render_func=lambda v: '%s' % str(v),
    )


register.snmp_section(
    name='cisco_asa_connections',
    parse_function=parse_cisco_asa_connections,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.9.9.147.1.2.2.2.1.5.40',  # CISCO-FIREWALL-MIB::cfwConnectionStatValue.protoIp
        oids=[
            '6',  # currentInUse
            '7',  # protoIp.high
        ]
    ),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.1.0', 'cisco adaptive security'),
        startswith('.1.3.6.1.2.1.1.1.0', 'cisco firepower threat defense'),
        startswith('.1.3.6.1.2.1.1.1.0', 'ccisco firewall services'),
        contains('.1.3.6.1.2.1.1.1.0', 'cisco pix security'),

    )
)

register.check_plugin(
    name='cisco_asa_connections',
    service_name='Firewall connections',
    discovery_function=discovery_cisco_asa_connections,
    check_function=check_cisco_asa_connections,
    check_default_parameters={},
    check_ruleset_name='cisco_fw_connections'
)
